@@ -19,6 +19,7 @@
  */
 
 #include <config.h>
+
 #include <gpgme.h>
 #include <gtk/gtk.h>
 #include "gpa.h"
@@ -100,13 +101,10 @@ gpa_expiry_dialog_run (GtkWidget * parent, gpgme_key_t key, GDate ** new_date)
   GtkWidget * vbox;
   GtkWidget * radio;
   GtkWidget * calendar;
-  GtkAccelGroup * accel_group;
-  unsigned long expiry_date;
+  time_t expiry_date;
 
   GPAExpiryDialog dialog;
 
-  accel_group = gtk_accel_group_new ();
-
   window = gtk_dialog_new_with_buttons (_("Change expiry date"),
                                         GTK_WINDOW (parent),
                                         GTK_DIALOG_MODAL,
@@ -118,19 +116,17 @@ gpa_expiry_dialog_run (GtkWidget * parent, gpgme_key_t key, GDate ** new_date)
   gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_OK);
   gtk_container_set_border_width (GTK_CONTAINER (window), 5);
   dialog.window = window;
-  accel_group = gtk_accel_group_new ();
-  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 
   vbox = GTK_DIALOG (window)->vbox;
   gtk_container_add (GTK_CONTAINER (window), vbox);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 
-  radio = gpa_radio_button_new (accel_group, _("_never expire"));
+  radio = gtk_radio_button_new_with_mnemonic (NULL, _("_never expire"));
   dialog.radio_never = radio;
   gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 0);
 
-  radio = gpa_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio),
-					    accel_group, _("_expire on:"));
+  radio = gtk_radio_button_new_with_mnemonic_from_widget
+    (GTK_RADIO_BUTTON (radio), _("_expire on:"));
   dialog.radio_date = radio;
   gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 0);
 
@@ -141,14 +137,14 @@ gpa_expiry_dialog_run (GtkWidget * parent, gpgme_key_t key, GDate ** new_date)
   gtk_box_pack_start (GTK_BOX (vbox), calendar, FALSE, FALSE, 0);
 
   g_signal_connect (G_OBJECT (dialog.radio_date), "toggled",
-                    (GCallback) expire_date_toggled_cb, calendar);
+                    G_CALLBACK (expire_date_toggled_cb), calendar);
 
-  expiry_date = key->subkeys->expires;
+  expiry_date = (time_t) key->subkeys->expires;
   
   if (expiry_date > 0)
     {
       GDate tmp;
-      g_date_set_time (&tmp, expiry_date);
+      g_date_set_time_t (&tmp, expiry_date);
       gtk_calendar_select_month (GTK_CALENDAR (calendar),
                                  g_date_get_month (&tmp)-1,
                                  g_date_get_year (&tmp));
