@@ -1,31 +1,27 @@
 /* fileman.c  -  The GNU Privacy Assistant
- *	Copyright (C) 2000, 2001 G-N-U GmbH.
- *      Copyright (C) 2007 g10 Code GmbH
- *
- * This file is part of GPA
- *
- * GPA is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * GPA is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
- */
+   Copyright (C) 2000, 2001 G-N-U GmbH.
+   Copyright (C) 2007, 2008 g10 Code GmbH
 
-/*
- *	The file encryption/decryption/sign window
- */
+   This file is part of GPA.
 
-#include "gpa.h"    /* Fixme - this does not belong here! */
-#include <config.h>
+   GPA is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+
+   GPA is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
+   License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+
+/* The file encryption/decryption/sign window.  */
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,6 +34,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
+#include "gpa.h"   
 #include "gpapastrings.h"
 
 #include "gtktools.h"
@@ -59,6 +56,7 @@
 #endif
 
 
+
 /* Object and class definition.  */
 struct _GpaFileManager
 {
@@ -66,7 +64,7 @@ struct _GpaFileManager
 
   GtkWidget *window;
   GtkWidget *list_files;
-  GList *selection_sensitive_widgets;
+  GList *selection_sensitive_actions;
 };
 
 struct _GpaFileManagerClass 
@@ -127,7 +125,7 @@ gpa_file_manager_finalize (GObject *object)
 static void
 gpa_file_manager_init (GpaFileManager *fileman)
 {
-  fileman->selection_sensitive_widgets = NULL;
+  fileman->selection_sensitive_actions = NULL;
 }
 
 static void
@@ -189,13 +187,17 @@ get_selected_files (GtkWidget *list)
 
   while (selection)
     {
+      gpa_file_item_t file_item;
       gchar *filename;
       GtkTreeIter iter;
 
       gtk_tree_model_get_iter (model, &iter, (GtkTreePath*) selection->data);
       gtk_tree_model_get (model, &iter, FILE_NAME_COLUMN, &filename, -1);
 
-      files = g_list_append (files, filename);
+      file_item = g_malloc0 (sizeof (*file_item));
+      file_item->filename_in = filename;
+
+      files = g_list_append (files, file_item);
       selection = g_list_next (selection);
     }
 
@@ -209,44 +211,50 @@ get_selected_files (GtkWidget *list)
 
 /* Add file FILENAME to the file list of FILEMAN and select it */
 static gboolean
-add_file (GpaFileManager *fileman, const char *filename)
+add_file (GpaFileManager *fileman, const gchar *filename)
 {
   GtkListStore *store;
   GtkTreeIter iter;
   GtkTreePath *path;
   GtkTreeSelection *select;
+  gchar *filename_utf8;
+
+  /* The tree contains filenames in the UTF-8 encoding.  */
+  filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL),
 
   store = GTK_LIST_STORE (gtk_tree_view_get_model
                           (GTK_TREE_VIEW (fileman->list_files)));
-  select = gtk_tree_view_get_selection (GTK_TREE_VIEW (fileman->list_files));
 
   /* Check for duplicates. */
   path = gtk_tree_path_new_first ();
-  gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
-  while (gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter))
-    {
-      gchar *tmp;
-      gboolean exists;
-
-      gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, FILE_NAME_COLUMN,
-			  &tmp, -1);
-      exists = g_str_equal (filename, tmp);
-      g_free (tmp);
-      if (exists)
-        return FALSE; /* This file is already in our list.  */
-    }
+  if (gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path))
+    do
+      {
+	gchar *tmp;
+	gboolean exists;
+	
+	gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, FILE_NAME_COLUMN,
+			    &tmp, -1);
+	exists = g_str_equal (filename_utf8, tmp);
+	g_free (tmp);
+	if (exists)
+	  {
+	    g_free (filename_utf8);
+	    gtk_tree_path_free (path);
+	    return FALSE; /* This file is already in our list.  */
+	  }
+      }
+    while (gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter));
   gtk_tree_path_free (path);
 
   /* Append it to our list.  */
   gtk_list_store_append (store, &iter);
 
   /* FIXME: Add the file status when/if gpgme supports it */
-  gtk_list_store_set (store, &iter,
-                      FILE_NAME_COLUMN,
-		      g_filename_to_utf8 (filename, -1, NULL, NULL, NULL),
-		      -1);
+  gtk_list_store_set (store, &iter, FILE_NAME_COLUMN, filename_utf8, -1);
   
   /* Select the row */
+  select = gtk_tree_view_get_selection (GTK_TREE_VIEW (fileman->list_files));
   gtk_tree_selection_select_iter (select, &iter);
 
   return TRUE;
@@ -255,11 +263,11 @@ add_file (GpaFileManager *fileman, const char *filename)
 
 /* Add a file created by an operation to the list */
 static void
-file_created_cb (GpaFileOperation *op, const gchar *filename, gpointer data)
+file_created_cb (GpaFileOperation *op, gpa_file_item_t item, gpointer data)
 {
   GpaFileManager *fileman = data;
   
-  add_file (fileman, filename);  
+  add_file (fileman, item->filename_out);
 }
 
 
@@ -275,9 +283,7 @@ register_operation (GpaFileManager *fileman, GpaFileOperation *op)
 
 
 
-/*
- * Management of the selection sensitive widgets;
- */
+/* Management of the selection sensitive actions.  */
 
 /* Return true if a selection is active.  */
 static gboolean
@@ -291,71 +297,110 @@ has_selection (gpointer param)
   return (gtk_tree_selection_count_selected_rows (select) > 0);
 }
 
-/* Add WIDGET to the list of sensitive widgets of FILEMAN.  */
+/* Add WIDGET to the list of sensitive actions of FILEMAN.  */
 static void
-add_selection_sensitive_widget (GpaFileManager *fileman,
-                                GtkWidget *widget,
-                                sensitivity_func_t callback)
+add_selection_sensitive_action (GpaFileManager *fileman,
+				 GtkAction *action,
+				 sensitivity_func_t callback)
 {
-  gtk_object_set_data (GTK_OBJECT (widget), "gpa_sensitivity", callback);
-  fileman->selection_sensitive_widgets \
-    = g_list_append(fileman->selection_sensitive_widgets, widget);
+  g_object_set_data (G_OBJECT (action), "gpa_sensitivity", callback);
+  fileman->selection_sensitive_actions
+    = g_list_append (fileman->selection_sensitive_actions, action);
 }
 
 /* Update the sensitivity of the widget DATA and pass PARAM through to
-   the sensitivity callback. Usable as an iterator function in
-   g_list_foreach. */
+   the sensitivity callback.  Usable as an iterator function in
+   g_list_foreach.  */
 static void
-update_selection_sensitive_widget (gpointer data, gpointer param)
+update_selection_sensitive_action (gpointer data, gpointer param)
 {
   sensitivity_func_t func;
 
-  func = gtk_object_get_data (GTK_OBJECT (data), "gpa_sensitivity");
-  gtk_widget_set_sensitive (GTK_WIDGET (data), func (param));
+  func = g_object_get_data (G_OBJECT (data), "gpa_sensitivity");
+  gtk_action_set_sensitive (GTK_ACTION (data), func (param));
 }
 
 
-/* Call update_selection_sensitive_widget for all widgets in the list
-   of sensitive widgets and pass FILEMAN through as the user data
+/* Call update_selection_sensitive_action for all widgets in the list
+   of sensitive actions and pass FILEMAN through as the user data
    parameter.  */
 static void
-update_selection_sensitive_widgets (GpaFileManager *fileman)
+update_selection_sensitive_actions (GpaFileManager *fileman)
 {
-  g_list_foreach (fileman->selection_sensitive_widgets,
-                  update_selection_sensitive_widget,
+  g_list_foreach (fileman->selection_sensitive_actions,
+                  update_selection_sensitive_action,
                   (gpointer) fileman);
 }
 
 
 
-/* 
-   Actions as called by the menu items. 
+/* Actions as called by the menu items.  */
 
-*/
+
+static GSList *
+get_load_file_name (GtkWidget *parent, const gchar *title,
+		    const gchar *directory)
+{
+  static GtkWidget *dialog;
+  GtkResponseType response;
+  GSList *filenames = NULL;
+
+  if (! dialog)
+    {
+      dialog = gtk_file_chooser_dialog_new
+	(title, GTK_WINDOW (parent), GTK_FILE_CHOOSER_ACTION_OPEN,
+	 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	 GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+      gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dialog), TRUE);
+    }
+  if (directory)
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), directory);
+  gtk_file_chooser_unselect_all (GTK_FILE_CHOOSER (dialog));
+  
+  /* Run the dialog until there is a valid response.  */
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  if (response == GTK_RESPONSE_OK)
+    filenames = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (dialog));
+
+  gtk_widget_hide (dialog);
+
+  return filenames;
+}
 
 
 /* Handle menu item "File/Open".  */
-static void
-open_file (gpointer param)
+
+void
+open_file_one (gpointer data, gpointer user_data)
 {
-  GpaFileManager * fileman = param;
-  gchar * filename;
+  GpaFileManager *fileman = user_data;
+  gchar *filename = (gchar *) data;
 
-  filename = gpa_get_load_file_name (GTK_WIDGET (fileman),
-                                     _("Open File"), NULL);
-  if (filename)
-    {
-
-      if (!add_file (fileman, filename))
-        gpa_window_error (_("The file is already open."),
-			    GTK_WIDGET (fileman));
-      g_free (filename);
-    }
+  /* FIXME: We are ignoring errors here.  */
+  add_file (fileman, filename);
+  g_free (filename);
 }
 
+
+static void
+file_open (GtkAction *action, gpointer param)
+{
+  GpaFileManager *fileman = param;
+  GSList *filenames;
+
+  filenames = get_load_file_name (GTK_WIDGET (fileman), _("Open File"), NULL);
+  if (! filenames)
+    return;
+
+  g_slist_foreach (filenames, open_file_one, fileman);
+  g_slist_free (filenames);
+}
+
+
 /* Handle menu item "File/Clear".  */
 static void
-close_all_files (gpointer param)
+file_clear (GtkAction *action, gpointer param)
 {
   GpaFileManager *fileman = param;
   GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model
@@ -367,10 +412,10 @@ close_all_files (gpointer param)
 
 /* Handle menu item "File/Verify".  */
 static void
-verify_files (gpointer param)
+file_verify (GtkAction *action, gpointer param)
 {
   GpaFileManager *fileman = param;
-  GList * files;
+  GList *files;
   GpaFileVerifyOperation *op;
 
   files = get_selected_files (fileman->list_files);
@@ -385,7 +430,7 @@ verify_files (gpointer param)
 
 /* Handle menu item "File/Sign".  */
 static void
-sign_files (gpointer param)
+file_sign (GtkAction *action, gpointer param)
 {
   GpaFileManager *fileman = param;
   GList * files;
@@ -395,7 +440,7 @@ sign_files (gpointer param)
   if (!files)
     return;
 
-  op = gpa_file_sign_operation_new (GTK_WIDGET (fileman), files);
+  op = gpa_file_sign_operation_new (GTK_WIDGET (fileman), files, FALSE);
 
   register_operation (fileman, GPA_FILE_OPERATION (op));
 }
@@ -403,17 +448,17 @@ sign_files (gpointer param)
 
 /* Handle menu item "File/Encrypt".  */
 static void
-encrypt_files (gpointer param)
+file_encrypt (GtkAction *action, gpointer param)
 {
   GpaFileManager *fileman = param;
-  GList * files;
+  GList *files;
   GpaFileEncryptOperation *op;
 
   files = get_selected_files (fileman->list_files);
   if (!files)
     return;
 
-  op = gpa_file_encrypt_operation_new (GTK_WIDGET (fileman), files);
+  op = gpa_file_encrypt_operation_new (GTK_WIDGET (fileman), files, FALSE);
 
   register_operation (fileman, GPA_FILE_OPERATION (op));
 }
@@ -421,17 +466,17 @@ encrypt_files (gpointer param)
 
 /* Handle menu item "File/Decrypt".  */
 static void
-decrypt_files (gpointer param)
+file_decrypt (GtkAction *action, gpointer param)
 {
   GpaFileManager *fileman = param;
-  GList * files;
+  GList *files;
   GpaFileDecryptOperation *op;
 
   files = get_selected_files (fileman->list_files);
   if (!files)
     return;
 
-  op = gpa_file_decrypt_operation_new (GTK_WIDGET (fileman), files);
+  op = gpa_file_decrypt_verify_operation_new (GTK_WIDGET (fileman), files);
 
   register_operation (fileman, GPA_FILE_OPERATION (op));
 }
@@ -439,7 +484,7 @@ decrypt_files (gpointer param)
 
 /* Handle menu item "File/Close".  */
 static void
-close_window (gpointer param)
+file_close (GtkAction *action, gpointer param)
 {
   GpaFileManager *fileman = param;
   gtk_widget_destroy (GTK_WIDGET (fileman));
@@ -448,7 +493,7 @@ close_window (gpointer param)
 
 /* Handle menu item "Edit/Select All".  */
 static void
-fileman_select_all (gpointer param)
+edit_select_all (GtkAction *action, gpointer param)
 {
   GpaFileManager *fileman = param;
   
@@ -458,222 +503,144 @@ fileman_select_all (gpointer param)
 
 
 
-/* Construct the file manager menu window and return that object. */
-static GtkWidget *
-fileman_menu_new (GpaFileManager *fileman)
+/* Construct the file manager menu and toolbar widgets and return
+   them. */
+static void
+fileman_action_new (GpaFileManager *fileman, GtkWidget **menubar,
+		    GtkWidget **toolbar)
 {
-  GtkItemFactory *factory;
-  GtkItemFactoryEntry file_menu[] = {
-    {_("/_File"), NULL, NULL, 0, "<Branch>"},
-    {_("/File/_Open"), NULL, open_file, 0, "<StockItem>", GTK_STOCK_OPEN},
-    {_("/File/C_lear"), NULL, close_all_files, 0, "<StockItem>", GTK_STOCK_CLEAR},
-    {_("/File/sep1"), NULL, NULL, 0, "<Separator>"},
-    {_("/File/_Sign"), NULL, sign_files, 0, NULL},
-    {_("/File/_Verify"), "<control>P", verify_files, 0, NULL},
-    {_("/File/_Encrypt"), NULL, encrypt_files, 0, NULL},
-    {_("/File/_Decrypt"), NULL, decrypt_files, 0, NULL},
-    {_("/File/sep2"), NULL, NULL, 0, "<Separator>"},
-    {_("/File/_Close"), NULL, close_window, 0, "<StockItem>", GTK_STOCK_CLOSE},
-    {_("/File/_Quit"), NULL, gtk_main_quit, 0, "<StockItem>", GTK_STOCK_QUIT},
-  };
-  GtkItemFactoryEntry edit_menu[] = {
-    {_("/_Edit"), NULL, NULL, 0, "<Branch>"},
-    {_("/Edit/Select _All"), "<control>A", fileman_select_all, 0, NULL},
-    {_("/Edit/sep2"), NULL, NULL, 0, "<Separator>"},
-    {_("/Edit/Pr_eferences..."), NULL, gpa_open_settings_dialog, 0,
-     "<StockItem>", GTK_STOCK_PREFERENCES},
-  };
-  GtkItemFactoryEntry windows_menu[] = {
-    {_("/_Windows"), NULL, NULL, 0, "<Branch>"},
-    {_("/Windows/_Filemanager"), NULL, gpa_open_filemanager, 0, NULL},
-    {_("/Windows/_Keyring Editor"), NULL, gpa_open_keyring_editor, 0, NULL},
-  };
+  static const GtkActionEntry entries[] =
+    {
+      /* Toplevel.  */
+      { "File", NULL, N_("_File"), NULL },
+      { "Edit", NULL, N_("_Edit"), NULL },
+
+      /* File menu.  */
+      { "FileOpen", GTK_STOCK_OPEN, NULL, NULL,
+	N_("Open a file"), G_CALLBACK (file_open) },
+      { "FileClear", GTK_STOCK_CLEAR, NULL, NULL,
+	N_("Close all files"), G_CALLBACK (file_clear) },
+      { "FileSign", GPA_STOCK_SIGN, NULL, NULL,
+	N_("Sign the selected file"), G_CALLBACK (file_sign) },
+      { "FileVerify", GPA_STOCK_VERIFY, NULL, NULL,
+	N_("Check signatures of selected file"), G_CALLBACK (file_verify) },
+      { "FileEncrypt", GPA_STOCK_ENCRYPT, NULL, NULL,
+	N_("Encrypt the selected file"), G_CALLBACK (file_encrypt) },
+      { "FileDecrypt", GPA_STOCK_DECRYPT, NULL, NULL,
+	N_("Decrypt the selected file"), G_CALLBACK (file_decrypt) },
+      { "FileClose", GTK_STOCK_CLOSE, NULL, NULL,
+	N_("Close the window"), G_CALLBACK (file_close) },
+      { "FileQuit", GTK_STOCK_QUIT, NULL, NULL,
+	N_("Quit the program"), G_CALLBACK (gtk_main_quit) },
+
+      /* Edit menu.  */
+      { "EditSelectAll", GTK_STOCK_SELECT_ALL, NULL, "<control>A",
+	N_("Select all files"), G_CALLBACK (edit_select_all) }
+    };
+
+  static const char *ui_description =
+    "<ui>"
+    "  <menubar name='MainMenu'>"
+    "    <menu action='File'>"
+    "      <menuitem action='FileOpen'/>"
+    "      <menuitem action='FileClear'/>"
+    "      <separator/>"
+    "      <menuitem action='FileSign'/>"
+    "      <menuitem action='FileVerify'/>"
+    "      <menuitem action='FileEncrypt'/>"
+    "      <menuitem action='FileDecrypt'/>"
+    "      <separator/>"
+    "      <menuitem action='FileClose'/>"
+    "      <menuitem action='FileQuit'/>"
+    "    </menu>"
+    "    <menu action='Edit'>"
+    "      <menuitem action='EditSelectAll'/>"
+    "      <separator/>"
+    "      <menuitem action='EditPreferences'/>"
+    "      <menuitem action='EditBackendPreferences'/>"
+    "    </menu>"
+    "    <menu action='Windows'>"
+    "      <menuitem action='WindowsKeyringEditor'/>"
+    "      <menuitem action='WindowsFileManager'/>"
+    "      <menuitem action='WindowsClipboard'/>"
+    "    </menu>"
+    "    <menu action='Help'>"
+#if 0
+    "      <menuitem action='HelpContents'/>"
+#endif
+    "      <menuitem action='HelpAbout'/>"
+    "    </menu>"
+    "  </menubar>"
+    "  <toolbar name='ToolBar'>"
+    "    <toolitem action='FileOpen'/>"
+    "    <toolitem action='FileClear'/>"
+    "    <separator/>"
+    "    <toolitem action='FileSign'/>"
+    "    <toolitem action='FileVerify'/>"
+    "    <toolitem action='FileEncrypt'/>"
+    "    <toolitem action='FileDecrypt'/>"
+    "    <separator/>"
+    "    <toolitem action='EditPreferences'/>"
+    "    <separator/>"
+    "    <toolitem action='WindowsKeyringEditor'/>"
+    "    <toolitem action='WindowsClipboard'/>"
+#if 0
+    "    <toolitem action='HelpContents'/>"
+#endif
+    "  </toolbar>"
+    "</ui>";
+
   GtkAccelGroup *accel_group;
-  GtkWidget *item;
+  GtkActionGroup *action_group;
+  GtkAction *action;
+  GtkUIManager *ui_manager;
+  GError *error;
 
-  accel_group = gtk_accel_group_new ();
-  factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
-  gtk_item_factory_create_items (factory,
-				 sizeof (file_menu) / sizeof (file_menu[0]),
-				 file_menu, fileman);
-  gtk_item_factory_create_items (factory,
-				 sizeof (edit_menu) / sizeof (edit_menu[0]),
-				 edit_menu, fileman);
-  gtk_item_factory_create_items (factory,
-				 sizeof(windows_menu) /sizeof(windows_menu[0]),
-				 windows_menu, fileman);
-
-  /* Disable buttons when no file is selected.   */
-  item = gtk_item_factory_get_widget (GTK_ITEM_FACTORY(factory),
-                                      _("/File/Sign"));
-  if (item)
-    add_selection_sensitive_widget (fileman, item, has_selection);
-
-  item = gtk_item_factory_get_widget (GTK_ITEM_FACTORY(factory),
-                                      _("/File/Verify"));
-  if (item)
-    add_selection_sensitive_widget (fileman, item, has_selection);
-
-  item = gtk_item_factory_get_widget (GTK_ITEM_FACTORY(factory),
-                                      _("/File/Encrypt"));
-  if (item)
-    add_selection_sensitive_widget (fileman, item, has_selection);
-
-  item = gtk_item_factory_get_widget (GTK_ITEM_FACTORY(factory),
-                                      _("/File/Decrypt"));
-  if (item)
-    add_selection_sensitive_widget (fileman, item, has_selection);
-
-  gpa_help_menu_add_to_factory (factory, GTK_WIDGET (fileman));
+  action_group = gtk_action_group_new ("MenuActions");
+  gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries),
+				fileman);
+  gtk_action_group_add_actions (action_group, gpa_help_menu_action_entries,
+				G_N_ELEMENTS (gpa_help_menu_action_entries),
+				fileman);
+  gtk_action_group_add_actions (action_group, gpa_windows_menu_action_entries,
+				G_N_ELEMENTS (gpa_windows_menu_action_entries),
+				fileman);
+  gtk_action_group_add_actions
+    (action_group, gpa_preferences_menu_action_entries,
+     G_N_ELEMENTS (gpa_preferences_menu_action_entries), fileman);
+  ui_manager = gtk_ui_manager_new ();
+  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+  accel_group = gtk_ui_manager_get_accel_group (ui_manager);
   gtk_window_add_accel_group (GTK_WINDOW (fileman), accel_group);
+  if (! gtk_ui_manager_add_ui_from_string (ui_manager, ui_description,
+					   -1, &error))
+    {
+      g_message ("building fileman menus failed: %s", error->message);
+      g_error_free (error);
+      exit (EXIT_FAILURE);
+    }
 
-  return gtk_item_factory_get_widget (factory, "<main>");
+  /* Fixup the icon theme labels which are too long for the toolbar.  */
+  action = gtk_action_group_get_action (action_group, "WindowsKeyringEditor");
+  g_object_set (action, "short_label", _("Keyring"), NULL);
+
+  /* Take care of sensitiveness of widgets.  */
+  action = gtk_action_group_get_action (action_group, "FileSign");
+  add_selection_sensitive_action (fileman, action, has_selection);
+  action = gtk_action_group_get_action (action_group, "FileVerify");
+  add_selection_sensitive_action (fileman, action, has_selection);
+  action = gtk_action_group_get_action (action_group, "FileEncrypt");
+  add_selection_sensitive_action (fileman, action, has_selection);
+  action = gtk_action_group_get_action (action_group, "FileDecrypt");
+  add_selection_sensitive_action (fileman, action, has_selection);
+
+  *menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
+  *toolbar = gtk_ui_manager_get_widget (ui_manager, "/ToolBar");
+  gpa_toolbar_set_homogeneous (GTK_TOOLBAR (*toolbar), FALSE);
 }
 
-
 
-/*
- * Toolbar actions.
- */
-
-static void
-toolbar_file_open (GtkWidget *widget, gpointer param)
-{
-  open_file (param);
-}
-
-static void
-toolbar_close_all (GtkWidget *widget, gpointer param)
-{
-  close_all_files (param);
-}
-
-static void
-toolbar_file_sign (GtkWidget *widget, gpointer param)
-{
-  sign_files (param);
-}
-
-static void
-toolbar_file_verify (GtkWidget *widget, gpointer param)
-{
-  verify_files (param);
-}
-
-static void
-toolbar_file_encrypt (GtkWidget *widget, gpointer param)
-{
-  encrypt_files (param);
-}
-
-static void
-toolbar_file_decrypt (GtkWidget *widget, gpointer param)
-{
-  decrypt_files (param);
-}
-
-#if 0
-static void
-toolbar_preferences (GtkWidget *widget, gpointer param)
-{
-  gpa_open_settings_dialog ();
-}
-#endif
-
-/* Construct the new toolbar object and return it.  Takes the file
-   manage object.  */
-static GtkWidget *
-fileman_toolbar_new (GpaFileManager *fileman)
-{
-  GtkWidget *toolbar, *icon, *item;
-
-  toolbar = gtk_toolbar_new ();
-  
-  /* Build the "Open" button.  */
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_OPEN,
-                            _("Open a file"), _("open file"),
-                            GTK_SIGNAL_FUNC (toolbar_file_open),
-                            fileman, -1);
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_CLEAR,
-                            _("Close all files"), _("close files"),
-                            GTK_SIGNAL_FUNC (toolbar_close_all),
-                            fileman, -1);
-  /* Build the "Sign" button.  */
-  if ((icon = gpa_create_icon_widget (GTK_WIDGET (fileman), "sign")))
-    {
-      item = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Sign"),
-				      _("Sign the selected file"),
-                                      _("sign file"),
-				      icon,
-				      GTK_SIGNAL_FUNC (toolbar_file_sign),
-				      fileman);
-      add_selection_sensitive_widget (fileman, item, has_selection);
-    }
-  /* Build the "Verify" button.  */
-  if ((icon = gpa_create_icon_widget (GTK_WIDGET (fileman), "verify")))
-    {
-      item = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Verify"),
-				      _("Check signatures of selected file"), 
-				      _("verify file"),
-                                      icon,
-				      GTK_SIGNAL_FUNC (toolbar_file_verify), 
-				      fileman);
-      add_selection_sensitive_widget (fileman, item, has_selection);
-    }
-  /* Build the "Encrypt" button.  */
-  if ((icon = gpa_create_icon_widget (GTK_WIDGET (fileman), "encrypt")))
-    {
-      item = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Encrypt"),
-				      _("Encrypt the selected file"),
-				      _("encrypt file"),
-				      icon,
-                                      GTK_SIGNAL_FUNC (toolbar_file_encrypt),
-				      fileman);
-      add_selection_sensitive_widget (fileman, item, has_selection);
-    }
-  /* Build the "Decrypt" button.  */
-  if ((icon = gpa_create_icon_widget (GTK_WIDGET (fileman), "decrypt")))
-    {
-      item = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Decrypt"),
-				      _("Decrypt the selected file"),
-				      _("decrypt file"),
-				      icon, 
-                                      GTK_SIGNAL_FUNC (toolbar_file_decrypt),
-				      fileman);
-      add_selection_sensitive_widget (fileman, item, has_selection);
-    }
-
-#if 0
-  /* Disabled for now. The long label causes the toolbar to grow too much.
-   * See http://bugzilla.gnome.org/show_bug.cgi?id=75068
-   */
-  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
-  
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), 
-                            GTK_STOCK_PREFERENCES,
-                            _("Open the Preferences dialog"),
-                            _("preferences"),
-                            GTK_SIGNAL_FUNC (toolbar_preferences),
-                            fileman, -1);
-#endif
-
-#if 0  /* FIXME: Help is not available yet. :-( */
-  /* Help */
-  if ((icon = gpa_create_icon_widget (GTK_WIDGET (fileman), "help")))
-    gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Help"),
-			     _("Understanding the GNU Privacy Assistant"),
-			     _("help"), icon,
-			     GTK_SIGNAL_FUNC (help_help), NULL);
-#endif
-
-  return toolbar;
-}
-
-
-
-/* 
-     Drag and Drop handler
- */
+/* Drag and Drop handler.  */
 
 
 /* Handler for "drag-drop".  This signal is emitted when the user
@@ -718,7 +685,7 @@ dnd_data_received_handler (GtkWidget *widget, GdkDragContext *context,
       /* Check that we got a format we can use.  */
       if (target_type == DND_TARGET_URI_LIST)
         {
-          char *p = selection_data->data;
+          char *p = (char *) selection_data->data;
           char **list;
           int i;
 
@@ -770,7 +737,7 @@ file_list_new (GpaFileManager * fileman)
   select = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
   gtk_tree_selection_set_mode (select, GTK_SELECTION_MULTIPLE);
   g_signal_connect_swapped (select, "changed",
-			    G_CALLBACK (update_selection_sensitive_widgets),
+			    G_CALLBACK (update_selection_sensitive_actions),
 			    fileman);
 
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (list), TRUE);
@@ -779,6 +746,8 @@ file_list_new (GpaFileManager * fileman)
   gtk_scrolled_window_set_policy  (GTK_SCROLLED_WINDOW (scrollerFile),
 				   GTK_POLICY_AUTOMATIC,
 				   GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollerFile),
+				       GTK_SHADOW_IN);
 
   fileman->list_files = list;
   gtk_widget_grab_focus (list);
@@ -805,10 +774,16 @@ gpa_file_manager_constructor (GType type,
   GObject *object;
   GpaFileManager *fileman;
   GtkWidget *vbox;
+  GtkWidget *hbox;
+  GtkWidget *label;
+  GtkWidget *icon;
+  gchar *markup;
   GtkWidget *menubar;
   GtkWidget *file_box;
   GtkWidget *file_frame;
   GtkWidget *toolbar;
+  GtkWidget *align;
+  guint pt, pb, pl, pr;
 
   /* Invoke parent's constructor.  */
   object = parent_class->constructor (type,
@@ -819,27 +794,48 @@ gpa_file_manager_constructor (GType type,
   /* Initialize.  */
   gtk_window_set_title (GTK_WINDOW (fileman),
 			_("GNU Privacy Assistant - File Manager"));
-  gtk_widget_set_usize (GTK_WIDGET (fileman), 640, 480);
+  gtk_window_set_default_size (GTK_WINDOW (fileman), 640, 480);
   /* Realize the window so that we can create pixmaps without warnings.  */
   gtk_widget_realize (GTK_WIDGET (fileman));
 
   /* Use a vbox to show the menu, toolbar and the file container.  */
   vbox = gtk_vbox_new (FALSE, 0);
 
-  /* First comes the menu.  */
-  menubar = fileman_menu_new (fileman);
+  /* Get the menu and the toolbar.  */
+  fileman_action_new (fileman, &menubar, &toolbar);
   gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, TRUE, 0);
-
-  /* Second the toolbar.  */
-  toolbar = fileman_toolbar_new(fileman);
   gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, TRUE, 0);
 
+
+  /* Add a fancy label that tells us: This is the file manager.  */
+  hbox = gtk_hbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 5);
+  
+  icon = gtk_image_new_from_stock ("gtk-directory", GTK_ICON_SIZE_DND);
+  gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, TRUE, 0);
+
+  label = gtk_label_new (NULL);
+  markup = g_strdup_printf ("<span font_desc=\"16\">%s</span>",
+                            _("File Manager"));
+  gtk_label_set_markup (GTK_LABEL (label), markup);
+  g_free (markup);
+  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 10);
+  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+
+
   /* Third a hbox with the file list.  */
+  align = gtk_alignment_new (0.5, 0.5, 1, 1);
+  gtk_alignment_get_padding (GTK_ALIGNMENT (align),
+			     &pt, &pb, &pl, &pr);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (align), pt, pb + 5,
+			     pl + 5, pr + 5);
+  gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
+
   file_box = gtk_hbox_new (TRUE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (file_box), 5);
   file_frame = file_list_new (fileman);
   gtk_box_pack_start (GTK_BOX (file_box), file_frame, TRUE, TRUE, 0);
-  gtk_box_pack_end (GTK_BOX (vbox), file_box, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (align), file_box);
+
   gtk_container_add (GTK_CONTAINER (fileman), vbox);
 
   g_signal_connect (object, "destroy",
@@ -869,7 +865,7 @@ gpa_fileman_new ()
   GpaFileManager *fileman;
 
   fileman = g_object_new (GPA_FILE_MANAGER_TYPE, NULL);  
-  update_selection_sensitive_widgets (fileman);
+  update_selection_sensitive_actions (fileman);
 
   return fileman;
 }
@@ -898,4 +894,5 @@ void gpa_file_manager_open_file (GpaFileManager *fileman,
   if (!add_file (fileman, filename))
     gpa_window_error (_("The file is already open."),
 		      GTK_WIDGET (fileman));
+  /* FIXME: Release filename?  */
 }
