@@ -18,6 +18,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include <config.h>
+
 #include <gpgme.h>
 #include <unistd.h>
 #include "gpa.h"
@@ -154,7 +156,7 @@ gpa_export_server_operation_get_destination (GpaExportOperation *operation,
       gpg_error_t err;
       *armor = TRUE;
       err = gpgme_data_new (dest);
-      if (gpg_err_code (err) != GPG_ERR_NO_ERROR)
+      if (err)
 	{
 	  gpa_gpgme_warning (err);
 	  return FALSE;    
