@@ -1,5 +1,6 @@
 /* gpafiledecryptop.c - The GpaOperation object.
- *	Copyright (C) 2003, Miguel Coca.
+ * Copyright (C) 2003 Miguel Coca.
+ * Copyright (C) 2008 g10 Code GmbH.
  *
  * This file is part of GPA
  *
@@ -18,7 +19,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include <config.h>
+
 #include <glib.h>
+#include <glib/gstdio.h>
 
 #ifdef G_OS_UNIX
 #include <unistd.h>
@@ -51,13 +55,56 @@ static void gpa_file_encrypt_operation_response_cb (GtkDialog *dialog,
 
 static GObjectClass *parent_class = NULL;
 
+/* Properties */
+enum
+{
+  PROP_0,
+  PROP_FORCE_ARMOR
+};
+
+
+static void
+gpa_file_encrypt_operation_get_property (GObject *object, guint prop_id,
+					 GValue *value, GParamSpec *pspec)
+{
+  GpaFileEncryptOperation *op = GPA_FILE_ENCRYPT_OPERATION (object);
+  
+  switch (prop_id)
+    {
+    case PROP_FORCE_ARMOR:
+      g_value_set_boolean (value, op->force_armor);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+gpa_file_encrypt_operation_set_property (GObject *object, guint prop_id,
+				      const GValue *value, GParamSpec *pspec)
+{
+  GpaFileEncryptOperation *op = GPA_FILE_ENCRYPT_OPERATION (object);
+
+  switch (prop_id)
+    {
+    case PROP_FORCE_ARMOR:
+      op->force_armor = g_value_get_boolean (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
 static void
 gpa_file_encrypt_operation_finalize (GObject *object)
 {  
   GpaFileEncryptOperation *op = GPA_FILE_ENCRYPT_OPERATION (object);
 
   /* FIXME: The use of RSET is messed up.  There is no clear concept
-     on who own the key.  This should be fixed by refing the keys
+     on who owns the key.  This should be fixed by refing the keys
      object.  I doubt that the keys are at all released. */
   g_free (op->rset);
   op->rset = NULL;
@@ -74,14 +121,15 @@ gpa_file_encrypt_operation_init (GpaFileEncryptOperation *op)
   op->plain_fd = -1;
   op->cipher = NULL;
   op->plain = NULL;
-  op->cipher_filename = NULL;
   op->encrypt_dialog = NULL;
+  op->force_armor = FALSE;
 }
 
 static GObject*
-gpa_file_encrypt_operation_constructor (GType type,
-					guint n_construct_properties,
-					GObjectConstructParam *construct_properties)
+gpa_file_encrypt_operation_constructor 
+	(GType type,
+         guint n_construct_properties,
+         GObjectConstructParam *construct_properties)
 {
   GObject *object;
   GpaFileEncryptOperation *op;
@@ -94,7 +142,7 @@ gpa_file_encrypt_operation_constructor (GType type,
   /* Initialize */
   /* Create the "Encrypt" dialog */
   op->encrypt_dialog = gpa_file_encrypt_dialog_new
-    (GPA_OPERATION (op)->window);
+    (GPA_OPERATION (op)->window, op->force_armor);
   g_signal_connect (G_OBJECT (op->encrypt_dialog), "response",
 		    G_CALLBACK (gpa_file_encrypt_operation_response_cb), op);
   /* Connect to the "done" signal */
@@ -120,8 +168,18 @@ gpa_file_encrypt_operation_class_init (GpaFileEncryptOperationClass *klass)
 
   object_class->constructor = gpa_file_encrypt_operation_constructor;
   object_class->finalize = gpa_file_encrypt_operation_finalize;
+  object_class->set_property = gpa_file_encrypt_operation_set_property;
+  object_class->get_property = gpa_file_encrypt_operation_get_property;
+
+  g_object_class_install_property (object_class,
+				   PROP_FORCE_ARMOR,
+				   g_param_spec_boolean
+				   ("force-armor", "Force armor",
+				    "Force armor mode", FALSE,
+				    G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
 
+
 GType
 gpa_file_encrypt_operation_get_type (void)
 {
@@ -153,19 +211,54 @@ gpa_file_encrypt_operation_get_type (void)
 /* API */
 
 GpaFileEncryptOperation*
-gpa_file_encrypt_operation_new (GtkWidget *window,
-				GList *files)
+gpa_file_encrypt_operation_new (GtkWidget *window, GList *files,
+				gboolean force_armor)
 {
   GpaFileEncryptOperation *op;
   
   op = g_object_new (GPA_FILE_ENCRYPT_OPERATION_TYPE,
 		     "window", window,
 		     "input_files", files,
+		     "force-armor", force_armor,
 		     NULL);
 
   return op;
 }
 
+
+GpaFileEncryptOperation*
+gpa_file_encrypt_sign_operation_new (GtkWidget *window, GList *files,
+				     gboolean force_armor)
+{
+  GpaFileEncryptOperation *op;
+  
+  op = g_object_new (GPA_FILE_ENCRYPT_OPERATION_TYPE,
+		     "window", window,
+		     "input_files", files,
+		     "force-armor", force_armor,
+		     NULL);
+
+  gpa_file_encrypt_dialog_set_sign
+    (GPA_FILE_ENCRYPT_DIALOG (op->encrypt_dialog), TRUE);
+
+  return op;
+}
+
+
+GpaFileEncryptOperation*
+gpa_file_encrypt_operation_new_for_server (GList *files, void *server_ctx)
+{
+  GpaFileEncryptOperation *op;
+  
+  op = g_object_new (GPA_FILE_ENCRYPT_OPERATION_TYPE,
+		     "input_files", files,
+                     "server-ctx", server_ctx,
+		     NULL);
+
+  return op;
+}
+
+
 /* Internal */
 
 static gchar*
@@ -187,75 +280,149 @@ destination_filename (const gchar *filename, gboolean armor)
   return cipher_filename;
 }
 
-static gboolean
+
+static gpg_error_t
 gpa_file_encrypt_operation_start (GpaFileEncryptOperation *op,
-				  const gchar *plain_filename)
+				  gpa_file_item_t file_item)
 {
   gpg_error_t err;
-  
-  op->cipher_filename = destination_filename 
-    (plain_filename, gpgme_get_armor (GPA_OPERATION (op)->context->ctx));
-  /* Open the files */
-  op->plain_fd = gpa_open_input (plain_filename, &op->plain, 
-				 GPA_OPERATION (op)->window);
-  if (op->plain_fd == -1)
+
+  if (file_item->direct_in)
     {
-      return FALSE;
-    }
-  op->cipher_fd = gpa_open_output (op->cipher_filename, &op->cipher,
-				   GPA_OPERATION (op)->window);
-  if (op->cipher_fd == -1)
-    {
-      gpgme_data_release (op->plain);
-      close (op->plain_fd);
-      return FALSE;
-    }
-  /* Start the operation */
-  /* Always trust keys, because any untrusted keys were already confirmed
-   * by the user.
-   */
-  if (gpa_file_encrypt_dialog_sign 
-      (GPA_FILE_ENCRYPT_DIALOG (op->encrypt_dialog)))
-    {
-      err = gpgme_op_encrypt_sign_start (GPA_OPERATION (op)->context->ctx,
-					 op->rset, GPGME_ENCRYPT_ALWAYS_TRUST,
-					 op->plain, op->cipher);
+      gpgme_error_t err;
+
+      /* No copy is made.  */
+      err = gpgme_data_new_from_mem (&op->plain, file_item->direct_in,
+				     file_item->direct_in_len, 0);
+      if (err)
+	{
+	  gpa_gpgme_warning (err);
+	  return err;
+	}
+      
+      err = gpgme_data_new (&op->cipher);
+      if (err)
+	{
+	  gpa_gpgme_warning (err);
+	  gpgme_data_release (op->plain);
+	  op->plain = NULL;
+	  return err;
+	}
     }
   else
     {
-      err = gpgme_op_encrypt_start (GPA_OPERATION (op)->context->ctx,
-				    op->rset, GPGME_ENCRYPT_ALWAYS_TRUST,
-				    op->plain, op->cipher);
+      gchar *plain_filename = file_item->filename_in;
+  
+      file_item->filename_out = destination_filename 
+	(plain_filename, gpgme_get_armor (GPA_OPERATION (op)->context->ctx));
+      /* Open the files */
+      op->plain_fd = gpa_open_input (plain_filename, &op->plain, 
+				     GPA_OPERATION (op)->window);
+      if (op->plain_fd == -1)
+	/* FIXME: Error value.  */
+	return gpg_error (GPG_ERR_GENERAL);
+
+      op->cipher_fd = gpa_open_output (file_item->filename_out, &op->cipher,
+				       GPA_OPERATION (op)->window);
+      if (op->cipher_fd == -1)
+	{
+	  gpgme_data_release (op->plain);
+	  close (op->plain_fd);
+	  op->plain_fd = -1;
+	  /* FIXME: Error value.  */
+	  return gpg_error (GPG_ERR_GENERAL);
+	}
     }
-  if (gpg_err_code (err) != GPG_ERR_NO_ERROR)
+
+  /* Start the operation.  */
+  /* Always trust keys, because any untrusted keys were already
+     confirmed by the user.  */
+  if (gpa_file_encrypt_dialog_get_sign 
+      (GPA_FILE_ENCRYPT_DIALOG (op->encrypt_dialog)))
+    err = gpgme_op_encrypt_sign_start (GPA_OPERATION (op)->context->ctx,
+				       op->rset, GPGME_ENCRYPT_ALWAYS_TRUST,
+				       op->plain, op->cipher);
+  else
+    err = gpgme_op_encrypt_start (GPA_OPERATION (op)->context->ctx,
+				  op->rset, GPGME_ENCRYPT_ALWAYS_TRUST,
+				  op->plain, op->cipher);
+
+  if (err)
     {
       gpa_gpgme_warning (err);
-      return FALSE;
+
+      gpgme_data_release (op->plain);
+      op->plain = NULL;
+      close (op->plain_fd);
+      op->plain_fd = -1;
+      gpgme_data_release (op->cipher);
+      op->cipher = NULL;
+      close (op->cipher_fd);
+      op->cipher_fd = -1;
+
+      return err;
     }
-  /* Show and update the progress dialog */
+
+  /* Show and update the progress dialog.  */
   gtk_widget_show_all (GPA_FILE_OPERATION (op)->progress_dialog);
   gpa_progress_dialog_set_label (GPA_PROGRESS_DIALOG 
 				 (GPA_FILE_OPERATION (op)->progress_dialog),
-				 op->cipher_filename);
-  return TRUE;
+				 file_item->direct_name
+				 ? file_item->direct_name
+				 : file_item->filename_in);
+  return 0;
 }
 
 static void
 gpa_file_encrypt_operation_next (GpaFileEncryptOperation *op)
 {
-  if (!GPA_FILE_OPERATION (op)->current ||
-      !gpa_file_encrypt_operation_start (op, GPA_FILE_OPERATION (op)
-					 ->current->data))
+  gpg_error_t err;
+
+  if (! GPA_FILE_OPERATION (op)->current)
     {
-      g_signal_emit_by_name (GPA_OPERATION (op), "completed");
+      g_signal_emit_by_name (GPA_OPERATION (op), "completed", 0);
+      return;
     }
+
+  err = gpa_file_encrypt_operation_start
+    (op, GPA_FILE_OPERATION (op)->current->data);
+  if (err)
+    g_signal_emit_by_name (GPA_OPERATION (op), "completed", err);
 }
 
+
 static void
 gpa_file_encrypt_operation_done_cb (GpaContext *context, 
 				    gpg_error_t err,
 				    GpaFileEncryptOperation *op)
 {
+  gpa_file_item_t file_item = GPA_FILE_OPERATION (op)->current->data;
+
+  if (file_item->direct_in)
+    {
+      size_t len;
+      char *cipher_gpgme = gpgme_data_release_and_get_mem (op->cipher,
+							   &len);
+      op->cipher = NULL;
+      /* Do the memory allocation dance.  */
+
+      if (cipher_gpgme)
+	{
+	  /* Conveniently make ASCII stuff into a string.  */
+	  file_item->direct_out = g_malloc (len + 1);
+	  memcpy (file_item->direct_out, cipher_gpgme, len);
+	  gpgme_free (cipher_gpgme);
+	  file_item->direct_out[len] = '\0';
+	  /* Yep, excluding the trailing zero.  */
+	  file_item->direct_out_len = len;
+	}
+      else
+	{
+	  file_item->direct_out = NULL;
+	  file_item->direct_out_len = 0;
+	}
+    }
+
   /* Do clean up on the operation */
   gpgme_data_release (op->plain);
   op->plain = NULL;
@@ -267,23 +434,23 @@ gpa_file_encrypt_operation_done_cb (GpaContext *context,
   op->cipher_fd = -1;
   gtk_widget_hide (GPA_FILE_OPERATION (op)->progress_dialog);
 
-  if (gpg_err_code (err) != GPG_ERR_NO_ERROR) 
+  if (err) 
     {
-      /* If an error happened, (or the user canceled) delete the created file
-       * and abort further encryptions
-       */
-      unlink (op->cipher_filename);
-      g_free (op->cipher_filename);
-      op->cipher_filename = NULL;
-      g_signal_emit_by_name (GPA_OPERATION (op), "completed");
+      if (! file_item->direct_in)
+	{
+	  /* If an error happened, (or the user canceled) delete the
+	    created file and abort further encryptions.  */
+	  g_unlink (file_item->filename_out);
+	  g_free (file_item->filename_out);
+	  file_item->filename_out = NULL;
+	}
+      g_signal_emit_by_name (GPA_OPERATION (op), "completed", err);
     }
   else
     {
       /* We've just created a file */
-      g_signal_emit_by_name (GPA_OPERATION (op), "created_file",
-			     op->cipher_filename);
-      g_free (op->cipher_filename);
-      op->cipher_filename = NULL;
+      g_signal_emit_by_name (GPA_OPERATION (op), "created_file", file_item);
+
       /* Go to the next file in the list and encrypt it */
       GPA_FILE_OPERATION (op)->current = g_list_next 
 	(GPA_FILE_OPERATION (op)->current);
@@ -322,7 +489,7 @@ ignore_key_trust (gpgme_key_t key, GtkWidget *parent)
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox);
 
-  label = gtk_label_new (_("You are going to encrypt a file using "
+  label = gtk_label_new (_("You are going to encrypt a document using "
 			   "the following key:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 5);
@@ -432,9 +599,33 @@ set_recipients (GpaFileEncryptOperation *op, GList *recipients)
 {
   GList *cur;
   int i;
+  gpgme_protocol_t protocol = GPGME_PROTOCOL_UNKNOWN;
 
+  g_free (op->rset);
   op->rset = g_malloc0 (sizeof(gpgme_key_t)*(g_list_length(recipients)+1));
 
+  /* Figure out the the protocol to use.  */
+  for (cur = recipients, i = 0; cur; cur = g_list_next (cur), i++)
+    {
+      gpgme_key_t key = cur->data;
+
+      if (protocol == GPGME_PROTOCOL_UNKNOWN)
+        protocol = key->protocol;
+      else if (key->protocol != protocol)
+        {
+          /* Should not happen either because the selection dialog
+             should have not allowed to select differet keys.  */
+          gpa_window_error 
+            (_("The selected certificates are not all of the same type."
+               " That is, you mixed OpenPGP and X.509 certificates."
+               " Please make sure to select only certificates of the"
+               " same type."), 
+             GPA_OPERATION (op)->window);
+          return FALSE;
+        }
+    }
+
+  /* Perform validity checks.  */
   for (cur = recipients, i = 0; cur; cur = g_list_next (cur), i++)
     {
       /* Check that all recipients are valid */
@@ -453,9 +644,13 @@ set_recipients (GpaFileEncryptOperation *op, GList *recipients)
           expired_key (key, GPA_OPERATION (op)->window);
           return FALSE;
         }
-      /* Now, check it's validity */
-      else if (valid == GPGME_VALIDITY_FULL || 
-               valid == GPGME_VALIDITY_ULTIMATE)
+      /* Now, check it's validity.  X.509 keys are always considered
+         valid becuase the backend will chekc this.  FIXME: It would
+         be better to ask the backend to check the validity of the key
+         instead of letting it fail later. */
+      else if (valid == GPGME_VALIDITY_FULL 
+               || valid == GPGME_VALIDITY_ULTIMATE
+               || key->protocol == GPGME_PROTOCOL_CMS)
 	{
 	  op->rset[i] = key;
 	}
@@ -478,6 +673,8 @@ set_recipients (GpaFileEncryptOperation *op, GList *recipients)
 	    }
 	}
     }
+
+  gpgme_set_protocol (GPA_OPERATION (op)->context->ctx, protocol);
   return TRUE;
 }
 
@@ -502,10 +699,8 @@ set_signers (GpaFileEncryptOperation *op, GList *signers)
     {
       gpgme_key_t key = cur->data;
       err = gpgme_signers_add (GPA_OPERATION (op)->context->ctx, key);
-      if (gpg_err_code (err) != GPG_ERR_NO_ERROR)
-	{
-	  gpa_gpgme_error (err);
-	}
+      if (err)
+	gpa_gpgme_error (err);
     }
   return TRUE;
 }
@@ -533,26 +728,21 @@ static void gpa_file_encrypt_operation_response_cb (GtkDialog *dialog,
       
       /* Set the armor value */
       gpgme_set_armor (GPA_OPERATION (op)->context->ctx, armor);
-      /* Set the signers for the context */
-      if (gpa_file_encrypt_dialog_sign 
+      /* Set the signers for the context.  */
+      if (gpa_file_encrypt_dialog_get_sign
 	  (GPA_FILE_ENCRYPT_DIALOG (op->encrypt_dialog)))
-	{
-	  success = set_signers (op, signers);
-	}
-      /* Set the recipients for the context */
+	success = set_signers (op, signers);
+
+      /* Set the recipients for the context.  */
       if (success)
-	{
-	  success = set_recipients (op, recipients);
-	}
-      /* Actually run the operation or abort */
+	success = set_recipients (op, recipients);
+
+      /* Actually run the operation or abort.  */
       if (success)
-	{
-	  gpa_file_encrypt_operation_next (op);
-	}
+	gpa_file_encrypt_operation_next (op);
       else
-	{
-	  g_signal_emit_by_name (GPA_OPERATION (op), "completed");
-	}
+	g_signal_emit_by_name (GPA_OPERATION (op), "completed",
+				 gpg_error (GPG_ERR_GENERAL));
 
       g_list_free (signers);
       g_list_free (recipients);
@@ -560,8 +750,9 @@ static void gpa_file_encrypt_operation_response_cb (GtkDialog *dialog,
   else
     {
       /* The dialog was canceled, so we do nothing and complete the
-       * operation */
-      g_signal_emit_by_name (GPA_OPERATION (op), "completed");
+	 operation */
+      g_signal_emit_by_name (GPA_OPERATION (op), "completed",
+			     gpg_error (GPG_ERR_CANCELED));
     }
 }
 
