@@ -1,31 +1,30 @@
-/* gpgmetools.h - additional gpgme support functions for GPA.
+/* gpgmetools.h - Additional gpgme support functions for GPA.
    Copyright (C) 2002 Miguel Coca.
-   Copyright (C) 2005 g10 Code GmbH.
+   Copyright (C) 2005, 2008 g10 Code GmbH.
 
-   This file is part of GPA.
+   This file is part of GPA
 
-   GPA is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   GPA is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
-   GPA is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GPA is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
+   License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GPA; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA  */
+   along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 /* A set of auxiliary functions for common tasks related to GPGME */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <errno.h>
+#include <ctype.h>
 #include "gpa.h"
 #include "gtktools.h"
 #include "gpgmetools.h"
@@ -43,6 +42,8 @@
 #include <windows.h>
 #endif
 
+#include <glib/gstdio.h>
+
 #ifndef O_BINARY
 #ifdef _O_BINARY
 #define O_BINARY	_O_BINARY
@@ -86,13 +87,14 @@ gpa_gpgme_new (void)
 {
   gpgme_ctx_t ctx;
   gpg_error_t err;
-  
+
+  /* g_assert (!"using gpa_gpgme_new"); */
   err = gpgme_new (&ctx);
   if (gpg_err_code (err) != GPG_ERR_NO_ERROR)
-    {
-      gpa_gpgme_error (err);
-    }
-  gpgme_set_passphrase_cb (ctx, gpa_passphrase_cb, NULL);
+    gpa_gpgme_error (err);
+
+  if (! cms_hack)
+    gpgme_set_passphrase_cb (ctx, gpa_passphrase_cb, NULL);
   
   return ctx;
 }
@@ -161,7 +163,7 @@ gpa_fopen (const char *filename, GtkWidget *parent)
   
   if (!check_overwriting (filename, parent))
     return NULL;
-  target = fopen (filename, "w");
+  target = g_fopen (filename, "w");
   if (!target)
     {
       gchar *message;
@@ -175,41 +177,49 @@ gpa_fopen (const char *filename, GtkWidget *parent)
 
 
 int
-gpa_open_output (const char *filename, gpgme_data_t *data, GtkWidget *parent)
+gpa_open_output_direct (const char *filename, gpgme_data_t *data,
+			GtkWidget *parent)
 {
   int target = -1;
-  
-  if (check_overwriting (filename, parent))
-    {
-      gpg_error_t err;
+  gpg_error_t err;
 
-      target = open (filename, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
-      if (target == -1)
-	{
-	  gchar *message;
-	  message = g_strdup_printf ("%s: %s", filename, strerror(errno));
-	  gpa_window_error (message, parent);
-	  g_free (message);
-	}
-      err = gpgme_data_new_from_fd (data, target);
-      if (gpg_err_code (err) != GPG_ERR_NO_ERROR)
-	{
-	  close (target);
-	  target = -1;
-	}
+  target = g_open (filename,
+		   O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
+  if (target == -1)
+    {
+      gchar *message;
+      message = g_strdup_printf ("%s: %s", filename, strerror(errno));
+      gpa_window_error (message, parent);
+      g_free (message);
+    }
+  err = gpgme_data_new_from_fd (data, target);
+  if (gpg_err_code (err) != GPG_ERR_NO_ERROR)
+    {
+      close (target);
+      target = -1;
     }
 
   return target;
 }
 
 
+int
+gpa_open_output (const char *filename, gpgme_data_t *data, GtkWidget *parent)
+{
+  if (! check_overwriting (filename, parent))
+    return -1;
+
+  return gpa_open_output_direct (filename, data, parent);
+}
+
+
 int
 gpa_open_input (const char *filename, gpgme_data_t *data, GtkWidget *parent)
 {
   gpg_error_t err;
   int target = -1;
 
-  target = open (filename, O_RDONLY | O_BINARY);
+  target = g_open (filename, O_RDONLY | O_BINARY, 0);
   if (target == -1)
     {
       gchar *message;
@@ -247,7 +257,34 @@ gpa_gpgme_data_new_from_file (gpgme_data_t *data,
   return err;
 }
 
-/* Write the contents of the gpgme_data_t into the clipboard.  */
+
+#ifdef G_OS_WIN32
+/* Convert newlines.  */
+void
+dos_to_unix (gchar *str, gsize *len)
+{
+  /* On Windows 2000, we need to convert \r\n to \n in the output for
+     cut & paste to work properly (otherwise, extra newlines will be
+     inserted).  */
+  gchar *src;
+  gchar *dst;
+  
+  src = str;
+  dst = str;
+  while (*src)
+    {
+      if (src[0] == '\r' && src[1] == '\n')
+	src++;
+      *(dst++) = *(src++);
+    }
+  *dst = '\0';
+  *len = dst - str;
+}
+#endif
+
+
+/* Write the contents of the gpgme_data_t into the clipboard.  Assumes
+   that the data is ASCII.  */
 void
 dump_data_to_clipboard (gpgme_data_t data, GtkClipboard *clipboard)
 {
@@ -273,6 +310,11 @@ dump_data_to_clipboard (gpgme_data_t data, GtkClipboard *clipboard)
       gpa_window_error (strerror (errno), NULL);
       exit (EXIT_FAILURE);
     }
+
+#ifdef G_OS_WIN32
+  dos_to_unix (text, &len);
+#endif
+
   gtk_clipboard_set_text (clipboard, text, len);
   g_free (text);
   return;
@@ -430,7 +472,7 @@ gpa_backup_key (const gchar *fpr, const char *filename)
   sec_argv[0] = (gchar*) path;
   /* Open the file */
   mask = umask (0077);
-  file = fopen (filename, "w");
+  file = g_fopen (filename, "w");
   umask (mask);
   if (!file)
     {
@@ -537,6 +579,9 @@ gpa_algorithm_from_string (const gchar * string)
 const gchar *
 gpa_key_ownertrust_string (gpgme_key_t key)
 {
+  if (key->protocol == GPGME_PROTOCOL_CMS)
+    return "";
+
   switch (key->owner_trust) 
     {
     case GPGME_VALIDITY_UNKNOWN:
@@ -655,11 +700,14 @@ gpa_passphrase_cb (void *hook, const char *uid_hint,
   
   dialog = gtk_dialog_new_with_buttons (_("Enter Passphrase"),
 					NULL, GTK_DIALOG_MODAL,
-					GTK_STOCK_OK,
-					GTK_RESPONSE_OK,
-                                        _("_Cancel"),
-					GTK_RESPONSE_CANCEL,
+                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					GTK_STOCK_OK, GTK_RESPONSE_OK,
 					NULL);
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
+
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, 
 		      TRUE, FALSE, 10);
@@ -976,3 +1024,194 @@ gpa_get_key_capabilities_text (gpgme_key_t key)
 	}
     }
 }
+
+
+/* Return a copy of the key array.  */
+gpgme_key_t *
+gpa_gpgme_copy_keyarray (gpgme_key_t *keys)
+{
+  gpgme_key_t *newarray;
+  int idx;
+
+  if (!keys)
+    return NULL;
+
+  for (idx=0; keys[idx]; idx++)
+    ;
+  idx++;
+  newarray = g_new (gpgme_key_t, idx); 
+  for (idx=0; keys[idx]; idx++)
+    {
+      gpgme_key_ref (keys[idx]);
+      newarray[idx] = keys[idx];
+    }
+  newarray[idx] = NULL;
+
+  return newarray;
+}
+
+
+/* Release all keys in the array KEYS as weel as ARRY itself.  */
+void
+gpa_gpgme_release_keyarray (gpgme_key_t *keys)
+{
+  if (keys)
+    {
+      int idx;
+      
+      for (idx=0; keys[idx]; idx++)
+        gpgme_key_unref (keys[idx]);
+      g_free (keys);
+    }
+}
+
+
+
+
+
+/* Read the next number in the version string STR and return it in
+   *NUMBER.  Return a pointer to the tail of STR after parsing, or
+   *NULL if the version string was invalid.  */
+static const char *
+parse_version_number (const char *str, int *number)
+{
+#define MAXVAL ((INT_MAX - 10) / 10)
+  int val = 0;
+
+  /* Leading zeros are not allowed.  */
+  if (*str == '0' && isascii (str[1]) && isdigit (str[1]))
+    return NULL;
+
+  while (isascii (*str) && isdigit (*str) && val <= MAXVAL)
+    {
+      val *= 10;
+      val += *(str++) - '0';
+    }
+  *number = val;
+  return val > MAXVAL ? NULL : str;
+#undef MAXVAL
+}
+
+
+/* Parse the version string STR in the format MAJOR.MINOR.MICRO (for
+   example, 9.3.2) and return the components in MAJOR, MINOR and MICRO
+   as integers.  The function returns the tail of the string that
+   follows the version number.  This might be te empty string if there
+   is nothing following the version number, or a patchlevel.  The
+   function returns NULL if the version string is not valid.  */
+static const char *
+parse_version_string (const char *str, int *major, int *minor, int *micro)
+{
+  str = parse_version_number (str, major);
+  if (!str || *str != '.')
+    return NULL;
+  str++;
+
+  str = parse_version_number (str, minor);
+  if (!str || *str != '.')
+    return NULL;
+  str++;
+
+  str = parse_version_number (str, micro);
+  if (!str)
+    return NULL;
+
+  /* A patchlevel might follow.  */
+  return str;
+}
+
+
+/* Return true if MY_VERSION is at least REQ_VERSION, and false
+   otherwise.  */
+static int
+compare_version_strings (const char *my_version,
+			 const char *rq_version)
+{
+  int my_major, my_minor, my_micro;
+  int rq_major, rq_minor, rq_micro;
+  const char *my_plvl, *rq_plvl;
+
+  if (!rq_version)
+    return 1;
+  if (!my_version)
+    return 0;
+
+  my_plvl = parse_version_string (my_version, &my_major, &my_minor, &my_micro);
+  if (!my_plvl)
+    return 0;
+
+  rq_plvl = parse_version_string (rq_version, &rq_major, &rq_minor, &rq_micro);
+  if (!rq_plvl)
+    return 0;
+
+  if (my_major > rq_major
+      || (my_major == rq_major && my_minor > rq_minor)
+      || (my_major == rq_major && my_minor == rq_minor 
+	  && my_micro > rq_micro)
+      || (my_major == rq_major && my_minor == rq_minor
+	  && my_micro == rq_micro && strcmp (my_plvl, rq_plvl) >= 0))
+    return 1;
+
+  return 0;
+}
+
+/* Try switching to the gpg2 backend.  */
+void
+gpa_switch_to_gpg2 (void)
+{
+  const char *oldname;
+  char *newname;
+  char *p;
+
+  if (is_gpg_version_at_least ("2.0.0"))
+    return; /* Already using 2.0.  */
+
+  oldname = get_gpg_path ();
+  g_return_if_fail (oldname && *oldname);
+  newname = xmalloc (strlen (oldname) + 1 + 1);
+  strcpy (newname, oldname);
+#ifdef G_OS_WIN32
+# define OLD_NAME "gpg.exe"  
+# define NEW_NAME "gpg2.exe"  
+  for (p=newname; *p; p++)
+    if (*p == '\\')
+      *p = '/';
+#else
+# define OLD_NAME "gpg"  
+# define NEW_NAME "gpg2"  
+#endif /*G_OS_WIN32*/
+  p = strrchr (newname, '/');
+  if (p)
+    p++;
+  if (!strcmp (p, OLD_NAME))
+    {
+      strcpy (p, NEW_NAME);
+      if (!access (newname, X_OK))
+        {
+          gpgme_set_engine_info (GPGME_PROTOCOL_OpenPGP, newname, NULL);
+          g_message ("switched engine to `%s'", newname);
+        }
+    }
+#undef OLD_NAME
+#undef NEW_NAME
+  xfree (newname);
+}
+
+
+/* Return 1 if the gpg engine has at least version NEED_VERSION,
+   otherwise 0.  */
+int
+is_gpg_version_at_least (const char *need_version)
+{
+  gpgme_engine_info_t engine;
+  
+  gpgme_get_engine_info (&engine);
+  while (engine)
+    {
+      if (engine->protocol == GPGME_PROTOCOL_OpenPGP)
+        return !!compare_version_strings (engine->version, need_version);
+      engine = engine->next;
+    }
+  return 0; /* No gpg-engine available. */
+}
+
